import { Injectable } from '@angular/core';
import { BehaviorSubject } from 'rxjs';

export interface Popup {
  title: string;
  message: string;
  type: 'win' | 'offer' | 'urgent' | 'fomo';
  cta: string;
  expires?: string;
  subMessage?: string;
  imageUrl?: string;
}

@Injectable({
  providedIn: 'root',
})
export class PopupService {
  private readonly POPUP_TEMPLATES: Popup[] = [
    {
      title: '🎯 VIP OFFER',
      message: 'Enhanced RTP + 500% Bonus on Next 5 Deposits',
      subMessage: 'Limited availability - 3 spots remaining',
      type: 'urgent',
      cta: 'Claim VIP Bonus',
      expires: '5:00',
    },
    {
      title: '🎰 BIG WIN ALERT',
      message: 'Recent win: €89,432 on minimum bet!',
      subMessage: 'Game is hot - Enhanced win rate active',
      type: 'win',
      cta: 'Play Now',
    },
  ];

  private readonly DISPLAY_CONFIG = {
    MIN_INTERVAL: 30000,
    AUTO_CLOSE_DELAY: 8000,
    SHOW_CHANCE: 0.7,
  };

  private readonly popupState = new BehaviorSubject<boolean>(false);
  private readonly currentPopup = new BehaviorSubject<Popup | null>(null);
  private lastPopupTime = 0;

  readonly showPopup$ = this.popupState.asObservable();
  readonly currentPopup$ = this.currentPopup.asObservable();

  showRandomPopup(): void {
    if (!this.shouldShowPopup()) return;

    const popup = this.getRandomPopup();
    this.displayPopup(popup);
    this.scheduleAutoClose();
    this.updateLastPopupTime();
  }

  showSpecificPopup(popup: Popup): void {
    if (!this.shouldShowPopup()) return;

    this.displayPopup(popup);
    this.updateLastPopupTime();
  }

  closePopup(): void {
    this.popupState.next(false);
  }

  private shouldShowPopup(): boolean {
    const now = Date.now();
    const timeSinceLastPopup = now - this.lastPopupTime;
    const isMinIntervalPassed = timeSinceLastPopup >= this.DISPLAY_CONFIG.MIN_INTERVAL;
    const isCurrentlyHidden = !this.popupState.value;
    const isRandomChanceSuccess = Math.random() <= this.DISPLAY_CONFIG.SHOW_CHANCE;

    return isMinIntervalPassed && isCurrentlyHidden && isRandomChanceSuccess;
  }

  private getRandomPopup(): Popup {
    const randomIndex = Math.floor(Math.random() * this.POPUP_TEMPLATES.length);
    return this.POPUP_TEMPLATES[randomIndex];
  }

  private displayPopup(popup: Popup): void {
    this.currentPopup.next(popup);
    this.popupState.next(true);
  }

  private scheduleAutoClose(): void {
    setTimeout(() => this.closePopup(), this.DISPLAY_CONFIG.AUTO_CLOSE_DELAY);
  }

  private updateLastPopupTime(): void {
    this.lastPopupTime = Date.now();
  }
}
