import { Injectable } from '@angular/core';
import { BehaviorSubject } from 'rxjs';

export interface Winner {
  name: string;
  amount: number;
  game: string;
  timestamp: Date;
  isVIP: boolean;
  betAmount?: number;
  multiplier?: number;
}

@Injectable({
  providedIn: 'root',
})
export class WinnerService {
  private readonly INITIAL_ONLINE_PLAYERS = 2547;
  private readonly TOTAL_PLAYERS_TODAY = 15789;
  private readonly TOTAL_WINNERS_TODAY = 12453;
  private readonly VIP_CHANCE = 0.3;
  private readonly MAX_RECENT_WINNERS = 10;

  private readonly PLAYER_NAMES = {
    FIRST: ['Alex', 'Maria', 'John', 'Sarah', 'Mike', 'Lisa', 'David', 'Emma'],
    LAST: ['K.', 'S.', 'M.', 'L.', 'R.', 'T.', 'B.', 'W.'],
  };

  private readonly ONLINE_PLAYERS_LIMITS = {
    MIN: 2000,
    MAX: 3500,
    CHANGE_RANGE: 15,
  };

  private readonly winners = new BehaviorSubject<Winner[]>([]);
  private readonly onlinePlayers = new BehaviorSubject<number>(this.INITIAL_ONLINE_PLAYERS);

  readonly recentWinners$ = this.winners.asObservable();
  readonly onlinePlayers$ = this.onlinePlayers.asObservable();

  constructor() {
    this.initializeWinners();
  }

  generateNewWinner(game: string, baseAmount: number): void {
    const winner = this.createWinner(game, baseAmount);
    this.updateWinnersList(winner);
  }

  updateOnlinePlayers(): void {
    const currentCount = this.onlinePlayers.value;
    const newCount = this.calculateNewPlayerCount(currentCount);
    this.onlinePlayers.next(newCount);
  }

  getTotalPlayersToday(): number {
    return this.TOTAL_PLAYERS_TODAY;
  }

  getTotalWinnersToday(): number {
    return this.TOTAL_WINNERS_TODAY;
  }

  private initializeWinners(): void {
    const initialWinners = [
      this.createWinner('Mega Fortune Dreams', 15432),
      this.createWinner('Lightning Roulette', 8745),
      this.createWinner('Golden Tiger', 12321),
    ];
    this.winners.next(initialWinners);
  }

  private createWinner(game: string, baseAmount: number): Winner {
    const betAmount = this.calculateBetAmount();
    const multiplier = Math.floor(baseAmount / betAmount);

    return {
      name: this.generateRandomName(),
      amount: baseAmount,
      game,
      timestamp: new Date(),
      isVIP: Math.random() > this.VIP_CHANCE,
      betAmount,
      multiplier,
    };
  }

  private calculateBetAmount(): number {
    return Math.floor(Math.random() * 100) + 10;
  }

  private updateWinnersList(winner: Winner): void {
    const currentWinners = this.winners.value;
    const updatedWinners = [winner, ...currentWinners];

    if (updatedWinners.length > this.MAX_RECENT_WINNERS) {
      updatedWinners.pop();
    }

    this.winners.next(updatedWinners);
  }

  private generateRandomName(): string {
    const firstName = this.getRandomArrayElement(this.PLAYER_NAMES.FIRST);
    const lastName = this.getRandomArrayElement(this.PLAYER_NAMES.LAST);
    return `${firstName} ${lastName}`;
  }

  private calculateNewPlayerCount(currentCount: number): number {
    const change = Math.floor(Math.random() * this.ONLINE_PLAYERS_LIMITS.CHANGE_RANGE) - 5;
    return Math.max(
      this.ONLINE_PLAYERS_LIMITS.MIN,
      Math.min(this.ONLINE_PLAYERS_LIMITS.MAX, currentCount + change)
    );
  }

  private getRandomArrayElement<T>(array: T[]): T {
    return array[Math.floor(Math.random() * array.length)];
  }
}
